require 'spec_helper'

describe "API" do

  before(:each) do
     @project = Project.create!(name: 'delinquo')
  end

  after(:each) do
    Project.delete_all
  end

  describe "Creating and listing projects" do

    it "can add a new project" do
      post '/project/cookiescrumbs'
      check_status 201
    end

    it "returns an error adding the same project twice" do
      post '/project/cookiescrumbs'
      post '/project/cookiescrumbs'
      check_status 409
      last_response.header['Warning'].should == 'project already exists with the name cookiescrumbs'
    end

    it "can list current projects" do
      get '/projects'
      check_status 200
      last_response.body.should == ["delinquo"].to_json
    end

    it "returns an error if a project doesnt exist" do
      get '/project/notaproject'
      check_status 404
      last_response.header['Warning'].should == 'project not found with the name notaproject'
    end

  end

  describe "Adding runs to a project" do
    
    it "needs to have a project name" do
      post '/project/james/run', json_for_a_run_with_two_fails
      check_status 404
      last_response.header['Warning'].should == 'project not found with the name james'
    end

    it "can add a run to a project" do
      add_run 'failing'
      @project.runs.where(total: '500').count.should == 1
    end

    it "can add more than one run to a project" do
      add_runs 2, 'failing'
      @project.runs.count.should == 2
    end

    it "can add a set of fails to an existing run" do
      add_run 'failing'
      @project.runs.first.fails.count.should == 2
    end

    it "can add a passing run" do
      add_run 'passing'
      @project.runs.first.fails.should == []
    end

  end

  describe "Returning runs from a project" do

      it "returns all runs for a project" do
        project = Project.where(name: 'delinquo').first
        10.times do
          project.runs.create! total: '500'
        end

        get '/project/delinquo'
        check_status 200
        JSON.parse(last_response.body).count.should == 10
      end

      it "can have a run with more than one fail" do
        post '/project/delinquo/run', json_for_a_run_with_two_fails
        check_status 201
        
        get '/project/delinquo'
        check_status 200

        json = JSON.parse(last_response.body)
        json[0]['fails'].count.should == 2
      end

      it "has the correct fields" do
        
        post '/project/delinquo/run', json_for_a_run_with_two_fails
        check_status 201

        get '/project/delinquo'
        check_status 200

        json = JSON.parse(last_response.body)

        json[0]['total'].should == '500'
        json[0]['fails'].first['feature'].should == 'a feature'
        json[0]['fails'].first['scenario'].should == 'a scenario'
        json[0]['fails'].first['line'].should == 'a line'
        json[0]['fails'].first['step'].should == 'a step'

        json[0]['id'].should be_nil
        json[0]['project_id'].should be_nil

        json[0]['fails'].first['id'].should be_nil
        json[0]['fails'].first['run_id'].should be_nil
      end

      it "rejects an empty run" do
        post '/project/delinquo/run', {}.to_json
        Project.where(name: 'delinquo').first.runs.count.should == 0
        check_status 400
      end

  end

  describe "numbers of runs per project" do

      it "contains a maximum of ten runs per project" do
        add_runs 11, 'failing'
        Project.where(name: 'delinquo').first.runs.count.should == 10
      end

      it "removes the oldest run when the limit is reached" do
        add_runs 10, 'failing'
        oldest_run_id = Project.where(name: 'delinquo').first.runs.first.id
        add_run 'failing'
        Run.where(id:oldest_run_id).should be_empty
      end
  end
  
end