require 'spec_helper'

describe 'Report' do

  before(:each) do
     @project = Project.create!(name: 'delinquo')
  end

  after(:each) do
    Project.delete_all
  end

  describe 'Project info' do

    it 'has the correct page title' do
      visit '/report/delinquo'
      page.title.should == "Delinquo"
    end

    it 'has the name of the project in an h1' do
      visit '/report/delinquo'
      page.should have_css('h1', text: 'Delinquo')
    end

  end

  describe 'warns when a project does not exist' do

    it 'should have a warning message' do
      visit '/report/cookies'
      page.should have_css('.not_a_project', text: "That is not a project.")
    end

    it 'should have a warning icon' do
      visit '/report/cookiescrumbs'
      page.find('#warning')[:src].should == '/images/umbrella.svg'
    end

  end

  describe 'Forcast icon' do

    it 'has a forecast icon' do  
      add_runs 10, 'passing'
      visit '/report/delinquo'
      page.should have_css('#forecast')
    end

    it 'has a sun icon when all runs passed' do
      add_runs 10, 'passing'
      visit '/report/delinquo'
      page.find('#forecast')[:src].should == '/images/sun.svg'
    end

    it 'has a changable icon when between one and three runs fail' do
      add_runs 9, 'passing'
      add_runs 1, 'failing'

      visit '/report/delinquo'
      page.find('#forecast')[:src].should == '/images/changeable.svg'
    end

    it 'has a cloud icon when between four and six runs fail' do
      add_runs 6, 'passing'
      add_runs 4, 'failing'

      visit '/report/delinquo'
      page.find('#forecast')[:src].should == '/images/cloud.svg'
    end

    it 'has a rain cloud icon when between seven and nine runs fail' do
      add_runs 3, 'passing'
      add_runs 7, 'failing'

      visit '/report/delinquo'
      page.find('#forecast')[:src].should == '/images/rain.svg'
    end

      it 'has a storm cloud icon when ten runs fail' do
      add_runs 10, 'failing'

      visit '/report/delinquo'
      page.find('#forecast')[:src].should == '/images/storm.svg'
    end

  end

  describe 'Top fail' do

    it 'has the correct top fail in the top fail section' do
      add_runs_with_a_top_fail
      visit '/report/delinquo'

      page.should have_css("#top_fail", text: 'this is the most fequently failing test')
    end
    
  end

  describe 'Last ten runs list' do

    it 'has the last ten runs' do
      add_runs 10

      visit '/report/delinquo'
      page.all('#runs a').count.should == 10
    end

    it 'each run has its own collapsable element' do
      add_runs 10

      visit '/report/delinquo'
      page.all('.accordion-body').each_with_index do |element,index|
        element[:id].should == 'run_' + (index + 1).to_s
      end
    end

    it 'each run has its own link that open / closes its collapsable element' do
      add_runs 10

      visit '/report/delinquo'
      page.all('.accordion-toggle').each_with_index do |element,index|
        element[:href].should == '#run_' + (index + 1).to_s
      end
    end

    it 'gives each passing run a passed title' do
      add_runs 10, 'passing'

      visit '/report/delinquo'
      page.all('.accordion-toggle span.status').each_with_index do |element|
        element.text.should == 'Passed'
      end
    end

    it 'gives each failing run a failed title' do
      add_runs 10

      visit '/report/delinquo'
      page.all('.accordion-toggle span.status').each_with_index do |element|
        element.text.should == 'Failed'
      end
    end

    it 'should show a feature, scenario, step and line in a failing runs collapsable element' do
      add_runs 1

      visit '/report/delinquo'
      page.find('.accordion-inner').should have_content('Feature: a feature Scenario: a scenario  Step: a step Line: a line')
    end

    it 'should not have a collapsable element for a passing run' do
      add_runs 1, 'passing'

      visit '/report/delinquo'
      page.should_not have_css('.accordion-body')
    end

    it 'should have a data and time for each run' do
      add_run
      run_id = @project.runs.first.id
      @project.runs.update(run_id, created_at: "2013-10-13 13:21:10")
      visit '/report/delinquo'
      page.should have_css('.date-time', :text => '13/10/2013 13.21')
    end

  end

  describe 'Progression bar' do

    it 'should have a bar with 100% green if the tests all passed in a run' do
      add_run 'passed'
      visit '/report/delinquo'

      page.find('.progress-bar-success').native.attributes['style'].value.should == "width: 100%"
      page.find('.progress-bar-danger').native.attributes['style'].value.should == "width: 0%"
    end

    it 'should have a progression bar with mostly green and some red when 2 tests fails out of 500' do
      add_run 'failing'

      visit '/report/delinquo'
      page.find('.progress-bar-success').native.attributes['style'].value.should == "width: 99%"
      page.find('.progress-bar-danger').native.attributes['style'].value.should == "width: 1%"
    end

  end

  describe 'Expander' do

    it 'should show the number of failing and passing tests' do
      add_run 'failing'

      visit '/report/delinquo'
      page.should have_css('.total-number-of-fails', text: '2')
      page.should have_css('.total-number-of-passes', text: '498')
    end

    it 'should show the full command to run all the failing test' do
      add_run 'failing'

      visit '/report/delinquo'
      page.should have_css('.run-fails', text: 'bundle exec cucumber a line')
    end
    
  end

 end